# $Id$

"""Real-Time Transport Protocol."""

import dpkt

# RFC 3550
INFO_PADDING	= 1 << 5
INFO_EXT	= 1 << 4
INFO_CSRC_MASK	= 0xf

TYPE_MARKER	= 1 << 7
TYPE_MASK	= 0x7f

# Payload types, RFC 3551
types = {
    0:('PCMU', 8000, 1),
    1:('1016', 8000, 1),
    2:('G721', 8000, 1),
    3:('GSM', 8000, 1),
    4:('G723', 8000, 1),
    5:('DVI4', 8000, 1),
    6:('DVI4', 16000, 1),
    7:('LPC', 8000, 1),
    8:('PCMA', 8000, 1),
    9:('G722', 8000, 1),
    10:('L16', 44100, 2),
    11:('L16', 44100, 1),
    12:('QCELP', 8000, 1),
    13:('CN', 8000, 1),
    14:('MPA', 90000, 1),
    15:('G728', 8000, 1),
    16:('DVI4', 11025, 1),
    17:('DVI4', 22050, 1),
    18:('G729', 8000, 1),
}

class RTP(dpkt.Packet):
    __hdr__ = (
        ('vinfo', 'B', 0),
        ('type', 'B', 0),
        ('seq', 'H', 0),
        ('tstamp', 'I', 0),
        ('ssrc', 'I', 0)
        )
    csrc = ''
    pad = ''
    
    def unpack(self, buf):
        dpkt.Packet.unpack(self, buf)
        csrc = self.vinfo & INFO_CSRC_MASK
        if csrc:
            n = 4 * csrc
            self.csrc, self.data = self.data[:n], self.data[n:]
